/* 
 * Copyright (C) 2004 Volker Stroebel <mmv1@planetpenguin.de>
 * 
 * Copyright (C) 1999-2001 Jasmin F. Patry
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "widget.h"

#include "debug.h"

#include "ui_mgr.h"

namespace pp {
	
Widget::Widget()
{
	UIMgr.add(this);	
	hasFocus=false;
}

Widget::~Widget()
{
	UIMgr.remove(this);
}

void
Widget::setEnabled(bool enabled)
{
	if ( this->enabled != enabled ) {
		UIMgr.setDirty();
    }

    this->enabled = enabled;

    if ( !enabled ) {
		this->clicked = false;
		this->focused = false;
    }
}

void
Widget::setVisible(bool visible)
{
    /* This check is necessary to prevent infinite mutual recursion */
    if ( this->active != visible ) {
		setActive( visible );
    }

    if ( !this->visible && visible ) {
		UIMgr.setDirty();
    } else if ( this->visible && !visible ) {
		UIMgr.setDirty();
    }
    this->visible = visible;
}

void 
Widget::setActive( bool active )
{
    if ( !this->active && active ) {
		UIMgr.setDirty();
    } else if ( this->active && !active ) {
		UIMgr.setDirty();
    }

    this->active = active;

    if ( !active ) {
		setVisible( false );
    }
}

void
Widget::performClickAction()
{
    signalClicked.Emit();
}
	
	
bool
Widget::isInBox(const int x, const int y) const
{
    return (bool) (
	( x >= position.x ) &&
	( x <= position.x + size.x ) &&
	( y >= position.y ) &&
	( y <= position.y + size.y ) );
}
	
void
Widget::mouseMotion(int x, int y)
{
    if ( !enabled ) {
		return;
    }

    if ( isInBox(x, y) ) {
		if ( focused == false ) {
			print_debug( DEBUG_UI, "Mouse entered button" );
		    focused = true;
		    UIMgr.setDirty();
		}
    } else {
		if ( focused == true ) {
			print_debug( DEBUG_UI, "Mouse left button" );
		    focused = false;
		    UIMgr.setDirty();
		}
    }
}
void
Widget::mouseDown(int button,int x, int y )
{
    if ( button != SDL_BUTTON_LEFT ) {
		return;
    }

    if ( !isInBox(x, y) ) {
		return;
    }

    if ( !enabled ) {
		return;
    }

    if ( clicked == false ) {
		print_debug( DEBUG_UI, "Button is down" );
		clicked = true;
		UIMgr.setDirty();
    }
}

void
Widget::mouseUp(int button, int x, int y )
{
	if ( button != SDL_BUTTON_LEFT ) {
		return;
	}

    if ( !enabled ) {
		return;
    }

    if ( !isInBox(x, y) ) {
		if ( clicked ) {
		    print_debug( DEBUG_UI, "Button is up (not clicked)" );
		    clicked = false;
		    UIMgr.setDirty();
		}
		return;
    } 

    if ( clicked ) {
		clicked = false;
		print_debug( DEBUG_UI, "Button was clicked" );
		performClickAction();
		UIMgr.setDirty();
    }
}

void
Widget::simulateMouseClick()
{
	performClickAction();
}

void
Widget::removeFocus()
{
	hasFocus=false;
}

void
Widget::setFocus(bool focus)
{
	if(focus!=hasFocus){
		UIMgr.grabFocus(this,focus);
		hasFocus=focus;	
	}
}




} //namespace pp
